<?php
/*
 * Plugin Name: Garnet Seller
 * Version: 1.0.7
 * Description: The plugin enables integration with the Garnet system.
 * Requires at least: 6.3
 * Requires PHP: 7.0
 * License: GPLv3 or later
 * License URI: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * Text Domain: garnet-seller
 */

defined('ABSPATH') or exit;

class Garnet_Seller
{

	static $optionName = 'garnet-seller_integration_key';
	static $baseUrl = 'https://garnet.center';
	static $apiCheckName = 'should_request_api_key';



	static function init()
	{
		error_log('INITTTT ...');

		self::createSettingsPage();
		self::createSettingsMenuLink();
		self::addSettingsLinkToPlugin();
		self::checkIntegrationKeyAction();

		self::showIntegrationNotice();



	}

	  static function generateApiKey()
	{
		error_log('Activation ...');
		add_option(self::$apiCheckName, true);
		add_action('admin_init', function () {
			// When the API key is created, the URL contains ?success=1, in that case we stop asking for permission
 			if(strpos($_SERVER['REQUEST_URI'], 'success=1') !== false ){
				update_option(self::$apiCheckName, false);
			}
			// Create an API key and send it to Garnet
			if (get_option(self::$apiCheckName)) {

				error_log('Activating .....');
				$params = [
					'app_name' => 'Garnet Seller',
					'scope' => 'read_write',
					'return_url' => home_url('/wp-admin/admin.php?page=garnet-seller-settings'), // redict to the plugin
					'callback_url' => 'https://garnet.center/api/woocommerce-app-seller/auth', // redict to the plugin
					'user_id' => '1'
				];
				exit(wp_redirect(home_url('/wc-auth/v1/authorize?' . http_build_query($params))));
			}
		});

	 
	}



	static function showIntegrationNotice()
	{

		$gsip_data = get_option(self::$optionName);

		if (!isset($gsip_data['key']) || empty($gsip_data['key'])) {
			add_action('admin_notices', function () {
				?>
				<div class="notice notice-warning">
					<p><?php echo esc_html_e('WooCommerce is not connected to a marketplace', 'garnet-seller'); ?></p>
				</div>
				<?php
			});
		}
	}

	static function checkIntegrationKeyAction()
	{

		add_filter('pre_update_option_' . self::$optionName, function ($value) {

			$gsip_data = get_option(self::$optionName);

			if (isset($gsip_data['data'])) {
				self::deactivateIntegration();
				return '';
			}

			$result = self::activateIntegrationKey($value['key']);

			if ($result) {
				$value['data'] = self::getIntegrationInformation();
			} else {
				$value = [];
			}

			return $value;

		}, 10, 3);

	}

	static function createSettingsPage()
	{
		self::generateApiKey();
		add_action('admin_init', function () {

			register_setting('garnet-seller', self::$optionName);

			$gsip_data = get_option(self::$optionName);

			if (!isset($gsip_data['key']) || empty($gsip_data['key'])) {
				self::notActivated();
			} else {
				self::activated();
			}

		});

	}

	static function notActivated()
	{

		add_settings_section(
			'garnet-seller_sttings_section',
			esc_html__('WooCommerce store is not integrated', 'garnet-seller'),
			'garnet_seller_sttings_section_callback',
			'garnet-seller'
		);

		function garnet_seller_sttings_section_callback($args)
		{
			?>
			<p id="<?php echo esc_attr($args['id']); ?>"><?php esc_html_e('Enter integration key', 'garnet-seller'); ?></p>
			<p>Request the marketplace to <a href="https://garnet.center/docs/integrations/woocommerce/" target="_blank">generate an
					integration key</a>.</p>
			<?php
		}

		$gsip_data = get_option(self::$optionName);

		add_settings_field(
			'garnet-seller_integration_key',
			esc_html__('Integration key', 'garnet-seller'),
			function ($args) {
				?>
			<input type="text" name="<?php echo esc_attr(self::$optionName) ?>[key]"
				id="<?php echo esc_attr(self::$optionName) ?>_key"
				value="<?php echo esc_attr((isset($gsip_data['key']) ? $gsip_data['key'] : '')) ?>"
				placeholder="gis_xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" />
			<?php
			},
			'garnet-seller',
			'garnet-seller_sttings_section',
			array(
				'label_for' => 'garnet-seller_field_pill',
				'class' => 'garnet-seller_row',
				'type' => 'input',
				'subtype' => 'text',
				'name' => 'garnet-seller_integration_key',
				'id' => 'garnet-seller_integration_key'
			)
		);

	}

	static function activated()
	{

		add_settings_section(
			'garnet-seller_sttings_section',
			esc_html__('WooCommerce store is integrated', 'garnet-seller'),
			'garnet_seller_sttings_section_callback',
			'garnet-seller'
		);

		function garnet_seller_sttings_section_callback($args)
		{
			$gsip_data = get_option(Garnet_Seller::$optionName);

			?>
			<p id="<?php echo esc_attr($args['id']); ?>"><?php esc_html_e('WooCommerce is connected to', 'garnet-seller'); ?> -
				<?php echo esc_html($gsip_data['data']['storeName']) ?>
			</p>
			<p><?php esc_html_e('Store Name', 'garnet-seller'); ?> - <?php echo esc_html($gsip_data['data']['storeName']) ?></p>
			<p><?php esc_html_e('Comission', 'garnet-seller'); ?> - <?php echo esc_html($gsip_data['data']['commission']) ?>%
			</p>
			<?php
		}

	}

	static function createSettingsMenuLink()
	{

		add_action('admin_menu', function () {
			add_menu_page(
				'Garnet Seller',
				'Garnet Seller',
				'manage_options',
				'garnet-seller-settings',
				function () {

					if (!current_user_can('manage_options')) {
						return;
					}

					settings_errors('garnet-seller_messages');

					$gsip_data = get_option(self::$optionName);

					?>
				<div class="wrap">
					<h1><?php echo esc_html(get_admin_page_title()); ?></h1>
					<form action="options.php" method="post">
						<?php

							settings_fields('garnet-seller');

							do_settings_sections('garnet-seller');

							if (!isset($gsip_data['key']) || empty($gsip_data['key'])) {
								submit_button('Connect to the marketplace');
							} else {
								submit_button('Disconnect');
							}

							?>
					</form>
				</div>
				<div class="wrap">
					<a href="<?php echo esc_attr(self::$baseUrl . '/docs/contact-us.html') ?>" target="_blank">Garnet’s support</a><br>
					<a href="<?php echo esc_attr(self::$baseUrl . '/docs/privacy-policy.html') ?>" target="_blank">Terms and
						conditions</a>
				</div>
				<?php
				}
			);
		});

	}

	static function addSettingsLinkToPlugin()
	{

		add_filter('plugin_action_links_garnet-seller/garnet-seller.php', function ($links) {
			$url = esc_url(
				add_query_arg(
					'page',
					'garnet-seller-settings',
					get_admin_url() . 'admin.php'
				)
			);

			$settings_link = "<a href='$url'>" . esc_html__('Settings') . '</a>';

			array_push(
				$links,
				$settings_link
			);
			return $links;
		});

	}

	static function getIntegrationInformation()
	{

		$result = self::doRequest(self::$baseUrl . '/api/woocommerce-app-seller/integrations');

		if ($result['code'] === 200) {
			return json_decode($result['body'], true);
		}

		return false;
	}

	static function activateIntegrationKey($key)
	{

		$data = [
			'hash' => $key
		];

		$result = self::doRequest(self::$baseUrl . '/api/woocommerce-app-seller/integrations', $data);

		if ($result['code'] === 204) {
			return true;
		}

		add_settings_error('garnet-seller_messages', 'garnet-seller_message', esc_html__('Incorrect integration key', 'garnet-seller'), 'error');

		return false;
	}

	static function deactivateIntegration()
	{

		$result = self::doRequest(self::$baseUrl . '/api/woocommerce-app-seller/integrations', []);

		if ($result['code'] === 204) {

			add_settings_error('garnet-seller_messages', 'garnet-seller_message', esc_html__('Disconnected from marketplace', 'garnet-seller'), 'success');

			return true;
		}

		return false;
	}

	static function doRequest($url, $data = false)
	{

		if (is_array($data)) {

			$response = wp_remote_post($url, [
				'method' => 'PUT',
				'body' => wp_json_encode($data),
				'headers' => [
					'Content-Type' => 'application/json'
				],
				'sslverify' => false
			]);

		} else {

			$response = wp_remote_get($url, [
				'method' => 'GET',
				'headers' => [
					'Content-Type: application/json'
				],
				'sslverify' => false
			]);
		}

		return [
			'code' => wp_remote_retrieve_response_code($response),
			'body' => wp_remote_retrieve_body($response)
		];

	}
}

Garnet_Seller::init();